// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use crate::ShortcutTrigger;
use glib::translate::*;

glib::wrapper! {
    #[doc(alias = "GtkNeverTrigger")]
    pub struct NeverTrigger(Object<ffi::GtkNeverTrigger, ffi::GtkNeverTriggerClass>) @extends ShortcutTrigger;

    match fn {
        type_ => || ffi::gtk_never_trigger_get_type(),
    }
}

impl NeverTrigger {
    #[doc(alias = "gtk_never_trigger_get")]
    pub fn get() -> NeverTrigger {
        assert_initialized_main_thread!();
        unsafe { from_glib_none(ffi::gtk_never_trigger_get()) }
    }
}
