// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

glib::wrapper! {
    #[doc(alias = "GdkX11AppLaunchContext")]
    pub struct X11AppLaunchContext(Object<ffi::GdkX11AppLaunchContext, ffi::GdkX11AppLaunchContextClass>) @extends gdk::AppLaunchContext, gio::AppLaunchContext;

    match fn {
        type_ => || ffi::gdk_x11_app_launch_context_get_type(),
    }
}

impl X11AppLaunchContext {}
